package simulation;

import metadata.FactoryInput;
import metadata.FactoryMetadata;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

public class Factory extends Building {
    private final FactoryMetadata metadata;
    private final Map<ComponentType, Integer> inputsCount = new HashMap<>();

    private boolean isProductionStarted;
    private long productionTicks = 0L;

    public Factory(int id, String type, int x, int y, FactoryMetadata metadata) {
        super(id, type, x, y);
        this.metadata = metadata;

        // Les usines sans entrées devraient toujours produire
        isProductionStarted = metadata.getInputs().size() == 0;
        if (isProductionStarted) {
            productionTicks = metadata.getProductionInterval();
        }
    }

    @Override
    public Optional<Component> update() {
        if (!isProductionStarted) return Optional.empty();

        productionTicks++;
        float productionInterval = metadata.getProductionInterval();
        if (productionTicks < productionInterval) {

            if (productionTicks / productionInterval >= (2 / 3f)) {
                state = BuildingState.FULL;
            } else if (productionTicks / productionInterval >= (1 / 3f)) {
                state = BuildingState.TWO_THIRD;
            } else {
                state = BuildingState.ONE_THIRD;
            }

            return Optional.empty();
        }

        if (metadata.getInputs().size() > 0) {
            isProductionStarted = false;
        }

        productionTicks = 0;
        state = BuildingState.EMPTY;
        Component newComponent = new Component(metadata.getOutput().getType());
        return Optional.of(newComponent);
    }

    @Override
    public void processInput(Component input) {
        int inputCount = 0;
        if (inputsCount.containsKey(input.getType())) {
            inputCount = inputsCount.get(input.getType());
        }

        inputsCount.put(input.getType(), inputCount + 1);

        if (hasEnoughComponents()) {
            startProduction();
        }
    }

    private boolean hasEnoughComponents() {
        for (FactoryInput input : metadata.getInputs()) {
            if (!inputsCount.containsKey(input.getType()) || inputsCount.get(input.getType()) < input.getQuantity()) {
                return false;
            }
        }

        return true;
    }

    private void startProduction() {
        for (FactoryInput input : metadata.getInputs()) {
            int inputCount = inputsCount.get(input.getType());
            inputsCount.put(input.getType(), inputCount - input.getQuantity());
        }

        isProductionStarted = true;
        productionTicks = 0;
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder("[");

        if (!inputsCount.isEmpty()) {
            for (ComponentType type : inputsCount.keySet()) {
                builder.append(type.toString())
                        .append(": ")
                        .append(inputsCount.get(type));
            }
        }

        if (isProductionStarted) {
            builder.append(" (interval: ")
                    .append(productionTicks)
                    .append(')');
        }

        builder.append(']');
        return builder.toString();
    }
}
