package view;

import configuration.SimulationConfiguration;
import configuration.SimulationConfigurationSingleton;
import configuration.SimulationData;
import metadata.BuildingMetadata;
import simulation.Component;
import simulation.*;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.io.Serial;
import java.util.*;

public class MainPanel extends JPanel {
    @Serial
    private static final long serialVersionUID = 1L;

    private final Map<String, Map<BuildingState, Image>> buildingIcons = new HashMap<>();
    private final Map<ComponentType, Image> componentsIcons = new HashMap<>();
    private final Map<Integer, Building> buildingsById = new HashMap<>();
    private final Collection<ComponentRoute> componentRoutes = new ArrayList<>();

    private SimulationConfiguration configuration;
    private SimulationData simulationData;

    public MainPanel() {
        try {
            componentsIcons.put(ComponentType.METAL, loadImage("src/ressources/metal.png"));
            componentsIcons.put(ComponentType.MOTOR, loadImage("src/ressources/moteur.png"));
            componentsIcons.put(ComponentType.PLANE, loadImage("src/ressources/avion.png"));
            componentsIcons.put(ComponentType.WING, loadImage("src/ressources/aile.png"));
        } catch (IOException e) {
            throw new RuntimeException("Could not load components icons", e);
        }
    }

    public void reloadSimulation() {
        configuration = SimulationConfigurationSingleton.getInstance().getConfiguration();
        simulationData = configuration.getSimulationData();

        buildingsById.clear();
        simulationData.getBuildings().forEach(b -> buildingsById.put(b.getId(), b));

        buildingIcons.clear();
        configuration.getMetadata().values().forEach(m -> {
            try {
                loadBuildingImages(m);
            } catch (IOException e) {
                System.err.println("Could not load simulation icons");
            }
        });
    }

    int x = 0;

    @Override
    public void paint(Graphics g) {
        super.paint(g);

        if (configuration == null) return;

        simulationData.getRoutes().forEach(r -> paintRoute(r, g));
        simulationData.getBuildings().forEach(b -> paintBuilding(b, g));

        Collection<ComponentRoute> removedComponentRoutes = new ArrayList<>();
        for (ComponentRoute route : componentRoutes) {
            paintComponentRoute(route, g);

            if (route.isTransitFinished()) {
                removedComponentRoutes.add(route);
                route.sendToOutputBuilding();
            }
        }

        // Prevent concurrency problems
        componentRoutes.removeAll(removedComponentRoutes);
    }

    private void paintBuilding(Building building, Graphics g) {
        building.update().ifPresent(component -> addComponentRoute(component, building));

        Image icon = buildingIcons.get(building.getType()).get(building.getState());
        g.drawImage(icon, building.getX(), building.getY(), null);

        g.drawString(building.toString(), building.getX() - 20, building.getY() - 5);
    }

    private void addComponentRoute(Component component, Building fromBuilding) {
        Route route = simulationData.getRoutes().stream()
                .filter(r -> r.getFrom() == fromBuilding.getId())
                .findFirst().get();

        Building toBuilding = buildingsById.get(route.getTo());
        componentRoutes.add(new ComponentRoute(component, toBuilding, fromBuilding.getX(), fromBuilding.getY()));
    }

    private void paintRoute(Route route, Graphics g) {
        Building fromBuilding = buildingsById.get(route.getFrom());
        Image fromIcon = buildingIcons.get(fromBuilding.getType()).get(fromBuilding.getState());

        Building toBuilding = buildingsById.get(route.getTo());
        Image toIcon = buildingIcons.get(toBuilding.getType()).get(fromBuilding.getState());

        g.drawLine(
                fromBuilding.getX() + fromIcon.getWidth(null) / 2,
                fromBuilding.getY() + fromIcon.getHeight(null) / 2,
                toBuilding.getX() + toIcon.getWidth(null) / 2,
                toBuilding.getY() + toIcon.getHeight(null) / 2
        );
    }

    private void paintComponentRoute(ComponentRoute route, Graphics g) {
        ComponentType componentType = route.getComponent().getType();
        Image componentIcon = componentsIcons.get(componentType);

        route.updatePosition();

        g.drawImage(componentIcon, route.getX(), route.getY(), null);
    }

    private void loadBuildingImages(BuildingMetadata metadata) throws IOException {
        Map<BuildingState, Image> statesIcons = new HashMap<>();
        for (BuildingState state : BuildingState.values()) {
            statesIcons.put(state, loadBuildingImage(metadata, state));
        }

        buildingIcons.put(metadata.getType(), statesIcons);
    }

    private Image loadBuildingImage(BuildingMetadata metadata, BuildingState state) throws IOException {
        String iconPath = metadata.getIconsPaths().get(state);
        return loadImage(iconPath);
    }

    private Image loadImage(String path) throws IOException {
        return ImageIO.read(new File(path));
    }
}
